from unittest.mock import MagicMock

class IssuersApiMock:

    def __init__(self):
        self.mock_create_client = MagicMock()
        self.mock_create_issuer = MagicMock()
        self.mock_create_policy = MagicMock()
        self.mock_create_policy_rule = MagicMock()
        self.mock_create_upstream_alias = MagicMock()
        self.mock_create_upstream_group_mapping = MagicMock()
        self.mock_delete_client = MagicMock()
        self.mock_delete_policy = MagicMock()
        self.mock_delete_policy_rule = MagicMock()
        self.mock_delete_root = MagicMock()
        self.mock_delete_upstream_alias = MagicMock()
        self.mock_delete_upstream_group_mapping = MagicMock()
        self.mock_get_client = MagicMock()
        self.mock_get_issuer = MagicMock()
        self.mock_get_policy = MagicMock()
        self.mock_get_policy_rule = MagicMock()
        self.mock_get_root = MagicMock()
        self.mock_get_upstream_alias = MagicMock()
        self.mock_get_upstream_group_mapping = MagicMock()
        self.mock_get_upstreams = MagicMock()
        self.mock_get_wellknown_issuer_info = MagicMock()
        self.mock_list_clients = MagicMock()
        self.mock_list_issuer_roots = MagicMock()
        self.mock_list_issuer_upstreams = MagicMock()
        self.mock_list_issuers = MagicMock()
        self.mock_list_policies = MagicMock()
        self.mock_list_policy_rules = MagicMock()
        self.mock_list_upstream_aliases = MagicMock()
        self.mock_list_upstream_group_mappings = MagicMock()
        self.mock_list_wellknown_issuer_info = MagicMock()
        self.mock_replace_client = MagicMock()
        self.mock_replace_issuer = MagicMock()
        self.mock_replace_policy = MagicMock()
        self.mock_replace_policy_rule = MagicMock()
        self.mock_replace_root = MagicMock()
        self.mock_replace_upstream_alias = MagicMock()
        self.mock_replace_upstream_group_mapping = MagicMock()
        self.mock_reset_service_account = MagicMock()
        self.mock_reset_to_default_policy = MagicMock()
        self.mock_set_policy = MagicMock()

    def create_client(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.create_client with MagicMock.
        """
        return self.mock_create_client(self, *args, **kwargs)

    def create_issuer(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.create_issuer with MagicMock.
        """
        return self.mock_create_issuer(self, *args, **kwargs)

    def create_policy(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.create_policy with MagicMock.
        """
        return self.mock_create_policy(self, *args, **kwargs)

    def create_policy_rule(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.create_policy_rule with MagicMock.
        """
        return self.mock_create_policy_rule(self, *args, **kwargs)

    def create_upstream_alias(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.create_upstream_alias with MagicMock.
        """
        return self.mock_create_upstream_alias(self, *args, **kwargs)

    def create_upstream_group_mapping(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.create_upstream_group_mapping with MagicMock.
        """
        return self.mock_create_upstream_group_mapping(self, *args, **kwargs)

    def delete_client(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.delete_client with MagicMock.
        """
        return self.mock_delete_client(self, *args, **kwargs)

    def delete_policy(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.delete_policy with MagicMock.
        """
        return self.mock_delete_policy(self, *args, **kwargs)

    def delete_policy_rule(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.delete_policy_rule with MagicMock.
        """
        return self.mock_delete_policy_rule(self, *args, **kwargs)

    def delete_root(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.delete_root with MagicMock.
        """
        return self.mock_delete_root(self, *args, **kwargs)

    def delete_upstream_alias(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.delete_upstream_alias with MagicMock.
        """
        return self.mock_delete_upstream_alias(self, *args, **kwargs)

    def delete_upstream_group_mapping(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.delete_upstream_group_mapping with MagicMock.
        """
        return self.mock_delete_upstream_group_mapping(self, *args, **kwargs)

    def get_client(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_client with MagicMock.
        """
        return self.mock_get_client(self, *args, **kwargs)

    def get_issuer(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_issuer with MagicMock.
        """
        return self.mock_get_issuer(self, *args, **kwargs)

    def get_policy(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_policy with MagicMock.
        """
        return self.mock_get_policy(self, *args, **kwargs)

    def get_policy_rule(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_policy_rule with MagicMock.
        """
        return self.mock_get_policy_rule(self, *args, **kwargs)

    def get_root(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_root with MagicMock.
        """
        return self.mock_get_root(self, *args, **kwargs)

    def get_upstream_alias(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_upstream_alias with MagicMock.
        """
        return self.mock_get_upstream_alias(self, *args, **kwargs)

    def get_upstream_group_mapping(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_upstream_group_mapping with MagicMock.
        """
        return self.mock_get_upstream_group_mapping(self, *args, **kwargs)

    def get_upstreams(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_upstreams with MagicMock.
        """
        return self.mock_get_upstreams(self, *args, **kwargs)

    def get_wellknown_issuer_info(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.get_wellknown_issuer_info with MagicMock.
        """
        return self.mock_get_wellknown_issuer_info(self, *args, **kwargs)

    def list_clients(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_clients with MagicMock.
        """
        return self.mock_list_clients(self, *args, **kwargs)

    def list_issuer_roots(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_issuer_roots with MagicMock.
        """
        return self.mock_list_issuer_roots(self, *args, **kwargs)

    def list_issuer_upstreams(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_issuer_upstreams with MagicMock.
        """
        return self.mock_list_issuer_upstreams(self, *args, **kwargs)

    def list_issuers(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_issuers with MagicMock.
        """
        return self.mock_list_issuers(self, *args, **kwargs)

    def list_policies(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_policies with MagicMock.
        """
        return self.mock_list_policies(self, *args, **kwargs)

    def list_policy_rules(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_policy_rules with MagicMock.
        """
        return self.mock_list_policy_rules(self, *args, **kwargs)

    def list_upstream_aliases(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_upstream_aliases with MagicMock.
        """
        return self.mock_list_upstream_aliases(self, *args, **kwargs)

    def list_upstream_group_mappings(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_upstream_group_mappings with MagicMock.
        """
        return self.mock_list_upstream_group_mappings(self, *args, **kwargs)

    def list_wellknown_issuer_info(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.list_wellknown_issuer_info with MagicMock.
        """
        return self.mock_list_wellknown_issuer_info(self, *args, **kwargs)

    def replace_client(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.replace_client with MagicMock.
        """
        return self.mock_replace_client(self, *args, **kwargs)

    def replace_issuer(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.replace_issuer with MagicMock.
        """
        return self.mock_replace_issuer(self, *args, **kwargs)

    def replace_policy(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.replace_policy with MagicMock.
        """
        return self.mock_replace_policy(self, *args, **kwargs)

    def replace_policy_rule(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.replace_policy_rule with MagicMock.
        """
        return self.mock_replace_policy_rule(self, *args, **kwargs)

    def replace_root(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.replace_root with MagicMock.
        """
        return self.mock_replace_root(self, *args, **kwargs)

    def replace_upstream_alias(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.replace_upstream_alias with MagicMock.
        """
        return self.mock_replace_upstream_alias(self, *args, **kwargs)

    def replace_upstream_group_mapping(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.replace_upstream_group_mapping with MagicMock.
        """
        return self.mock_replace_upstream_group_mapping(self, *args, **kwargs)

    def reset_service_account(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.reset_service_account with MagicMock.
        """
        return self.mock_reset_service_account(self, *args, **kwargs)

    def reset_to_default_policy(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.reset_to_default_policy with MagicMock.
        """
        return self.mock_reset_to_default_policy(self, *args, **kwargs)

    def set_policy(self, *args, **kwargs):
        """
        This method mocks the original api IssuersApi.set_policy with MagicMock.
        """
        return self.mock_set_policy(self, *args, **kwargs)




from . import context

import agilicus

from .input_helpers import strip_none

from .output.table import (
    spec_column,
    format_table,
    metadata_column,
)


def list_feature_tags(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.features_api.list_feature_tags(**strip_none(kwargs)).feature_tags


def add_feature_tag(ctx, name, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    tag_spec = agilicus.FeatureTagSpec(name=agilicus.FeatureTagName(name))
    tag = agilicus.FeatureTag(spec=tag_spec)
    return apiclient.features_api.add_feature_tag(tag)


def show_feature_tag(ctx, name):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.features_api.get_feature_tag(feature_tag_name=name)


def delete_feature_tag(ctx, name):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.features_api.delete_feature_tag(feature_tag_name=name)


def format_feature_tags_as_text(ctx, tags):
    columns = [
        spec_column("name"),
        metadata_column("created"),
    ]

    return format_table(ctx, tags, columns)



from unittest.mock import MagicMock

class PolicyApiMock:

    def __init__(self):
        self.mock_get_challenge_decision = MagicMock()
        self.mock_get_enrollment_decision = MagicMock()
        self.mock_map_attributes = MagicMock()

    def get_challenge_decision(self, *args, **kwargs):
        """
        This method mocks the original api PolicyApi.get_challenge_decision with MagicMock.
        """
        return self.mock_get_challenge_decision(self, *args, **kwargs)

    def get_enrollment_decision(self, *args, **kwargs):
        """
        This method mocks the original api PolicyApi.get_enrollment_decision with MagicMock.
        """
        return self.mock_get_enrollment_decision(self, *args, **kwargs)

    def map_attributes(self, *args, **kwargs):
        """
        This method mocks the original api PolicyApi.map_attributes with MagicMock.
        """
        return self.mock_map_attributes(self, *args, **kwargs)




from .. import context
from ..input_helpers import (
    get_org_from_input_or_ctx,
    strip_none,
    build_updated_model_validate,
)
import agilicus
from ..output.table import (
    spec_column,
    format_table,
    metadata_column,
    column,
    subtable,
)


def add_label(ctx, name, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    spec = agilicus.LabelSpec(name=agilicus.LabelName(name), **kwargs)

    label = agilicus.Label(spec=spec)
    return apiclient.labels_api.create_object_label(label).to_dict()


def list_labels(ctx, name, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    if name is not None:
        kwargs["label_name"] = agilicus.LabelName(name)

    return apiclient.labels_api.list_object_labels(**kwargs).labels


def get_label(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    return apiclient.labels_api.get_object_label(**kwargs)


def delete_label(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    apiclient.labels_api.delete_object_label(**kwargs)


def replace_label(ctx, label_id, org_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    org_id = get_org_from_input_or_ctx(ctx, org_id=org_id)
    kwargs = strip_none(kwargs)

    label = apiclient.labels_api.get_object_label(label_id=label_id, org_id=org_id)
    kwargs["org_id"] = org_id

    label.spec = build_updated_model_validate(
        agilicus.LabelSpec,
        label.spec,
        kwargs,
        True,
    )

    return apiclient.labels_api.replace_object_label(label_id, label=label)


def format_labels(ctx, labels):
    columns = [
        metadata_column("id"),
        spec_column("org_id"),
        spec_column("name"),
        spec_column("description"),
    ]

    return format_table(ctx, labels, columns)


def add_labelled_object(ctx, label, object_type, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    if object_type is not None:
        kwargs["object_type"] = agilicus.ObjectType(object_type)

    labels = _label_associations_to_list(label, kwargs["org_id"])
    obj = agilicus.LabelledObject(labels=labels, **kwargs)

    return apiclient.labels_api.create_labelled_object(obj).to_dict()


def list_labelled_objects(
    ctx, includes_any_label, excludes_any_label, object_type, **kwargs
):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    if includes_any_label is not None:
        kwargs["includes_any_label"] = _labels_to_list(includes_any_label)
    if excludes_any_label is not None:
        kwargs["excludes_any_label"] = _labels_to_list(excludes_any_label)
    if object_type is not None:
        kwargs["object_type"] = agilicus.ObjectType(object_type)

    return apiclient.labels_api.list_labelled_objects(**kwargs).labelled_objects


def _label_associations_to_list(labels, org_id):
    return [
        agilicus.LabelAssociation(
            label_name=agilicus.LabelName(label),
            org_id=org_id,
        )
        for label in labels
    ]


def _labels_to_list(labels):
    return [agilicus.LabelName(label) for label in labels]


def get_labelled_object(ctx, object_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    return apiclient.labels_api.get_labelled_object(
        labelled_object_id=object_id, **kwargs
    )


def delete_labelled_object(ctx, object_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    apiclient.labels_api.delete_labelled_object(labelled_object_id=object_id, **kwargs)


def replace_labelled_object(ctx, label, object_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    obj = apiclient.labels_api.get_labelled_object(
        labelled_object_id=object_id, **kwargs
    )
    if not label:
        return obj

    obj.labels = _label_associations_to_list(label, kwargs["org_id"])
    return apiclient.labels_api.replace_labelled_object(
        obj.object_id, labelled_object=obj
    )


def format_labelled_objects(ctx, objs):
    labels = [
        column("label_name"),
    ]

    columns = [
        column("org_id"),
        column("object_id"),
        column("object_type"),
        subtable(ctx, "labels", labels),
    ]

    return format_table(ctx, objs, columns)


def add_labelled_object_label(ctx, label, object_id, org_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    org_id = get_org_from_input_or_ctx(ctx, org_id=org_id)
    assoc = agilicus.LabelAssociation(
        label_name=agilicus.LabelName(label), org_id=org_id
    )

    return apiclient.labels_api.create_labelled_object_label(object_id, assoc, **kwargs)


def delete_labelled_object_label(ctx, label, object_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    return apiclient.labels_api.delete_labelled_object_label(
        labelled_object_id=object_id, label_name=label, **kwargs
    )
